"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.pickRelated = void 0;
const sift_1 = require("sift");
const common_1 = require("@nestjs/common");
const util_1 = require("util");
const util = require("util");
const logger = new common_1.Logger("SeedRelation");
async function pickRelated(context, idColumns, entities, pick, many = false) {
    let picked;
    if (pick === undefined) {
        picked = entities[Math.floor(Math.random() * entities.length)];
    }
    else if (Array.isArray(pick)) {
        picked = await Promise.all(pick.map((v) => pickRelated(context, idColumns, entities, v, many)));
        if (many) {
            picked = Array.prototype.concat(...picked);
        }
    }
    else if (typeof pick === "function") {
        picked = pick(context, entities);
    }
    else if (isQuery(pick)) {
        picked = entities.filter((0, sift_1.default)(pick));
        if (many) {
            return picked;
        }
        else if (picked.length == 0) {
            return null;
        }
        else {
            logger.warn(`${picked.length} found for to-one relationship with query ${(0, util_1.inspect)(pick)}.`);
            return picked[0];
        }
    }
    else if (typeof pick === "number") {
        picked = entities[pick];
    }
    else {
        ((_) => { })(pick);
    }
    try {
        return restrict(idColumns, picked);
    }
    catch (e) {
        throw new Error(`Picked invalid relationship object(s): ${util.inspect(picked)}`);
    }
}
exports.pickRelated = pickRelated;
function restrict(cols, pick) {
    if (Array.isArray(pick))
        return pick.map((v) => restrict(cols, v));
    else if (pick === null || pick === undefined)
        return null;
    return Object.fromEntries(Object.entries(pick).filter((kv) => cols.includes(kv[0])));
}
function isQuery(arg) {
    return typeof arg === "object";
}
